/*
** Additional routines used by setpe/checkip
**
** Copyright (C) 2004  Moreno 'baro' Baricevic
**                                      <baro AT democritos DOT it>
**
** This  file may be used subject to the terms and conditions of the
** GNU Library General Public License as published by the Free Soft-
** ware Foundation; either version 2 of the License, or (at your op-
** tion) any later version.
**
** This file is distributed in the hope that it will be useful,  but
** WITHOUT  ANY  WARRANTY; without even the implied warranty of MER-
** CHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Li-
** brary General Public License for more details.
*/

/**************************************************
 * Author: Moreno 'baro' Baricevic                *
 * Date:   22 Oct 2004                            *
 * Title:  checkip.c                              *
 *================================================*
 * Prev Modified: 22 Oct 2004                     *
 * Last Modified: 23 Oct 2004                     *
 **************************************************/

#include "checkip.h"



/***********************************************************************
 *_____________________________  XCMALLOC _____________________________*
 ***********************************************************************/
/* (om)procps/proc/ominfo.c */
/*
** allocates zeroed memory buffer.
** Dies only on system failure (ENOMEM), not bugs(?) like size<=0.
*/
static void * xcmalloc ( const size_t size )
{

   errno = 0;

   if ( size > 0 )
   {
      register void * ptr = malloc( size );
      if ( ptr == NULL )
      {
         fprintf( stderr , "%s: allocation error, size = %d\n"
                         , __FUNCTION__ , size );
         exit( 1 );
      }
      return memset( ptr , 0 , size );
   }

   errno = EINVAL;

   return NULL;

} /* xcmalloc */



/********************************************************************
 *_____________________________  XFREE _____________________________*
 ********************************************************************/
/* (om)procps/proc/ominfo.c */
/*
** free and clean the pointer
*/
static void xfree ( void ** ptr )
{
   if ( ptr && *ptr )
   {
      free( *ptr );
      *ptr = NULL;
   }
} /* xfree */



/*****************************************************************************
 *_____________________________  GET_IFACE_INFO _____________________________*
 *****************************************************************************/
/*
** query (trough the given socket) an interface request structure for the
** network interface "iface". Store the acquired data into an already
** allocated if_s structure pointed by "buf".
*/
static int get_iface_info ( int sock , const char * iface , if_s * buf )
{
   struct ifreq ifreq;
   struct sockaddr_in *saptr = NULL;

   /* store iface name */
   snprintf( buf->iface , IF_NAMESIZE , "%s" , iface );

   /* copy in the interface name to look up address of */
   strncpy( ifreq.ifr_name , iface , IF_NAMESIZE );

   /* get the address for this interface */
   if ( ioctl( sock , SIOCGIFADDR , &ifreq ) != 0 )
   {
      fprintf( stderr , "ioctl failed on SIOCGIFADDR"
                        " for device '%s': [%d] %m\n"
                      , iface , errno );
      return -1 ;
   }

   saptr = (struct sockaddr_in *)&ifreq.ifr_addr;

   /* store iface IP addr (network byte order) */
   buf->naddr = saptr->sin_addr.s_addr;

   /* store iface IP addr (host byte order) */
   buf->haddr = ntohl( saptr->sin_addr.s_addr );

   /* store iface IP addr in DQN */
   snprintf( buf->ip , IP_LEN , QUADFMT , NIPQUAD( saptr->sin_addr.s_addr ) );

   return 0;

} /* get_iface_info */



/*********************************************************************
 *_____________________________  IP2STR _____________________________*
 *********************************************************************/
/*
** translate the IP from network byte order address to Dotted Quad
** Notation string (dinamically allocated).
** It's a memory leak if the returned pointer is never freed.
*/
char * nip2str( ulong nip )
{
   char * ipstr = (char *) xcmalloc( IP_LEN ) ;
   snprintf( ipstr , IP_LEN , QUADFMT , NIPQUAD( nip ) );
   return ipstr;
} /* nip2str */

/*
** translate the IP from host byte order address to Dotted Quad
** Notation string (dinamically allocated).
** It's a memory leak if the returned pointer is never freed.
*/
char * hip2str( ulong hip )
{
   char * ipstr = (char *) xcmalloc( IP_LEN ) ;
   snprintf( ipstr , IP_LEN , QUADFMT , HIPQUAD( hip ) );
   return ipstr;
} /* hip2str */



/************************************************************************
 *_____________________________  CLEAN_IFS _____________________________*
 ************************************************************************/
/*
** clean and free the chain structure dinamically allocated by get_myips()
*/
void clean_ifs( if_s * ptr )
{
   if_s * nextp;

   while ( ptr && ptr->next )
   {
      nextp = ptr->next;
      XFREE( ptr );
      ptr = nextp;
   }
} /* clean_ifs */



/************************************************************************
 *_____________________________  GET_MYIPS _____________________________*
 ************************************************************************/
/*
** returns a pointer to an if_s chain structure which contains the info
** about all the network interfaces (loopback excluded). The structure
** is empty (but not NULL) if no network interface has been found (the
** network is down).
** The structure is dinamically allacted and must be freed with
** clean_ifs().
*/
if_s * get_myips( void )
{
   if_s * ptr = new( if_s );
   if_s * walk = ptr , * old = ptr;

   struct if_nameindex *iflist = NULL, *listsave = NULL;

   int sock = socket( AF_INET , SOCK_STREAM , 0 );
   if ( sock < 0 )
   {
      fprintf( stderr , "socket failed: [%d] %m\n" , errno );
      exit( 1 );
   }

   /* Return a list of all interfaces and their indices. */
   iflist = listsave = if_nameindex();

   for ( ; *(char *)iflist != 0 ; iflist++ )
   {
      if ( strncmp( "lo" , iflist->if_name , 3 ) == 0 )
        continue;
      old = walk;
      if ( get_iface_info( sock , iflist->if_name , walk ) == 0 )
      {
         walk->next = new( if_s );
         walk = walk->next;
      }
   }
   if ( walk != ptr && ! walk->naddr )
   {
      XFREE( walk );
      old->next = NULL;
   }

   /* Free the data returned from if_nameindex. */
   if_freenameindex( listsave );

   close( sock );

   return ptr;

} /* get_myips */



/**********************************************************************
 *_____________________________  GET_HIP _____________________________*
 **********************************************************************/
/*
** returns the ip address for "hostname" in host byte order
*/
ulong get_hip( const char * hostname )
{
   struct hostent * hostent;
   ulong addr = 0;
   if ( ( hostent = gethostbyname( hostname ) ) == NULL )
   {
      if ( ( addr = inet_addr( hostname ) ) == -1 )
      {
         herror( hostname );
         return -1;
      }
   }
   else
   {
      bcopy( hostent->h_addr , &addr , sizeof( addr ) );
      addr = ntohl( addr );
   }
   DBGprint( "%s\n" , hip2str( addr ) );
   return addr;

} /* get_hip */



#ifdef DEBUG_CHECKIP
int main( void )
{
   if_s * ips = get_myips();
   if ( ips )
   {
      if_s * walk = ips;
      while ( walk && walk->naddr )
      {
         if ( strncmp( "lo" , walk->iface , 3 ) )
           printf( "%s %-15s %08lX %08lX [%p]\n"
                   , walk->iface , walk->ip
                   , walk->naddr , walk->haddr
                   , walk
                 );
         walk = walk->next;
      }
      clean_ifs( ips );
   }
   return 0;
}
#endif



/***********************  E N D   O F   F I L E  ************************/

